import java.io.PrintStream;
import java.util.Date;

import type.lib.CreditCard;
import type.lib.RewardCard;

/**
 * @author mb
 * 
 */
public class L07App05 {

	/**
	 * @param args
	 */
	public static void main(String[] args) {

		PrintStream output = System.out;
		CreditCard cc, rc;

		cc = new CreditCard(112233, "Charles Oakley", 5000);
		rc = new RewardCard(112234, "Tim Duncan", 6000); // this demonstrates
															// the
															// substitutability
															// principle

		cc.charge(1000);
		rc.charge(1000);

		output.println(cc.toString());
		output.println(rc.toString());

		// take home point #1: rc can be declared as a CreditCard at compile
		// time,
		// but still be
		// assigned a reference to a RewardCard at run time.
		//
		// take home point #2: which version of toString() is invoked on rc?
		// RewardCard's version or CreditCard's version?
		//
		//
		// The cool thing is that somehow the "right" version of the toString
		// was invoked. How did this happen? The VM knows that rc is referring
		// to a RewardCard object and thus looks in that class definition first
		// for a method called toString() (the late-binding part).
		// At compile time, the compiler wants to enforce the condition that any
		// method invoked on rc be valid. To the compiler, rc is a CreditCard.
		// The compiler is happy to find that every method that is invoked on rc
		// can be found in the CreditCard class definition (early-binding)
		//
		// This demonstrates that toString() is polymorphic - the same signature
		// actually simultaneously represents two different methods (one method
		// at compile time and another method at run time).

		/*
		 * toString is polymorphic because to the compiler, toString in line 31
		 * is seen as CreditCard's method (since that is the declared type of
		 * rc), but at runtime, the actual type of rc is RewardCard, so it is
		 * RewardCard's toString method that gets used.
		 * 
		 * This example L07App05 shows toString as polymorphic, but not
		 * L07App01... L07App04. why? because the declared type of rc here is
		 * CreditCard, whereas earlier the declared type was RewardCard
		 */

	}
}
